from act.types.format import object_format


def test_tool_format() -> None:
    assert object_format("tool", "Mimikatz") == "mimikatz"
    assert object_format("tool", "Super (awesome) tool") == "super tool"


def test_threatactor_format() -> None:
    assert object_format("threatActor", "APT28") == "apt28"
    assert object_format("threatActor", "APT28 ") == "apt28"
    assert object_format("threatActor", "supER+@ta") == "super+@ta"


def test_vulnerability_format() -> None:
    assert object_format("vulnerability", "CVE-2017-0199") == "cve-2017-0199"


def test_hash_format() -> None:
    sha256 = "87428fc522803d31065e7bce3cf03fe475096631e5e07bbd7a0fde60c4cf25c7"
    sha256_upper = sha256.upper()
    ssdeep = "24:Ol9rFBzwjx5ZKvBF+bi8RuM4Pp6rG5Yg+q8wIXhMC:qrFBzKx5s8sM4grq8wIXht"
    ssdeep_upper = "24:87428FC522803D31065E7BCE3CF03FE475096631E5E07BBD7A0FDE60C4CF25C7"
    assert object_format("content", sha256) == sha256
    assert object_format("content", sha256_upper) == sha256
    assert object_format("hash", sha256) == sha256
    assert object_format("hash", sha256_upper) == sha256

    # ssdeep mixed case and contains non hex digests and should never be lowercased
    assert object_format("hash", ssdeep) == ssdeep
    assert object_format("hash", ssdeep_upper) == ssdeep_upper



import pytest

from act.types.types import object_validates

PLACEHOLDER = (
    "[placeholder[d12da485a7ae4437499b0d5a97f71c5d33e1a90588717b020f9b1f4c18368be0]]"
)


def test_validator_non_string():
    with pytest.raises(TypeError):
        object_validates("tool", 1)

    with pytest.raises(TypeError):
        object_validates("tool", None)


def test_tool_validator():
    assert object_validates("tool", "Mimikatz") is False
    assert object_validates("tool", "mimikatz") is True
    assert object_validates("tool", "many 0days: ie") is True


def test_threatactor_validator():
    assert object_validates("threatActor", "APT28") is False
    assert object_validates("threatActor", "apT28") is False
    assert object_validates("threatActor", "apt28") is True
    assert object_validates("threatActor", "some-ta") is True
    assert object_validates("threatActor", "some-ta@!") is True
    assert object_validates("threatActor", PLACEHOLDER) is True


def test_uri_validator():
    # Illegal URIs
    assert object_validates("uri", "abc") is False

    # Legal URIs
    assert object_validates("uri", "http://www.mnemonic.no") is True
    assert object_validates("uri", "https://www.mnemonic.no") is True
    assert object_validates("uri", "https://www.mnemonic.no/abc?a=2") is True
    assert object_validates("uri", "https://www.mnemonic.no/a-b-c?a=2") is True
    assert object_validates("uri", "ftp://127.0.0.1") is True

    assert object_validates("uri", PLACEHOLDER) is True



#!/usr/bin/env python3

import argparse
import datetime
import os
import pickle
import urllib.parse
from typing import Generator, Optional, Tuple

import graphviz
import requests

from atlassian import Confluence


class DataModel:

    DEBUG = False

    def __init__(
        self,
        url: str,
        username: Optional[str] = None,
        password: Optional[str] = None,
        user_id: int = 0,
        cacert: str = "",
    ) -> None:
        self.objects_url = urllib.parse.urljoin(url, "/v1/objectType")
        self.facts_url = urllib.parse.urljoin(url, "/v1/factType")
        self.username = username
        self.password = password
        self.user_id = user_id
        self.status: Optional[int] = None
        self._objects: Optional[dict] = None
        self._facts: Optional[dict] = None
        self.verify = cacert if cacert != "" else True

    def load(self) -> None:
        """Download the datamodel from the act instance"""

        headers = {"ACT-User-ID": str(self.user_id), "Accept": "application/json"}
        auth = (self.username, self.password)

        if self.username:
            r = requests.get(
                self.objects_url, auth=auth, headers=headers, verify=self.verify
            )
        else:
            r = requests.get(self.objects_url, headers=headers, verify=self.verify)

        if r.status_code == 200:
            self._objects = r.json()
        else:
            if self.DEBUG:
                print("Error loading objects: {}".format(r.status_code))
            self._objects = None
            self._facts = None
            self.status = r.status_code
            return

        if self.username:
            r = requests.get(
                self.facts_url, auth=auth, headers=headers, verify=self.verify
            )
        else:
            r = requests.get(self.facts_url, headers=headers, verify=self.verify)

        if r.status_code == 200:
            self._facts = r.json()
        else:
            if self.DEBUG:
                print("Error loading objects: {}".format(r.status_code))
            self._objects = None
            self._facts = None
            self.status = r.status_code
            return

        self.status = r.status_code

    def __eq__(self, other: object) -> bool:
        return set(self.objects) == set(other.objects) and set(self.facts) == set(
            other.facts
        )

    @property
    def facts(self) -> Generator[Tuple[str, str, str, bool], None, None]:
        """Iterate over fact bindings"""

        if not self._facts:
            if self.DEBUG:
                print("Trying to iterate over None facts")
            return
        for fact in self._facts["data"]:
            if not fact:
                print("Nonetype", fact)
                continue
            bindings = fact["relevantObjectBindings"]
            if not bindings:
                continue
            for binding in bindings:
                yield (
                    fact["name"],
                    binding["sourceObjectType"]["name"],
                    binding["destinationObjectType"]["name"]
                    if binding["destinationObjectType"]
                    else None,
                    binding["bidirectionalBinding"],
                )

    @property
    def objects(self) -> Generator[str, None, None]:
        """Iterate over object bindings"""

        if not self._objects:
            if self.DEBUG:
                print("Trying to iterate over None objects")
            return

        for obj in self._objects["data"]:
            if not obj:
                print("Nonetype", obj)
                continue
            yield obj["name"]


def parse_args() -> argparse.Namespace:
    """Handle command line arguments, returning the arguments ns"""

    parser = argparse.ArgumentParser(description="Build a graph of the act datamodel")
    parser.add_argument("url", type=str, help="Url of the act instance to graph")
    parser.add_argument("--uid", default=1, type=int, help="Act user ID")
    parser.add_argument("--http_username", type=str, default=None, help="HTTP Username")
    parser.add_argument("--http_password", type=str, default=None, help="HTTP Password")
    parser.add_argument("--parent_id", type=int, default=None, help="Confluence upload")
    parser.add_argument(
        "--confluence_url", type=str, default=None, help="Confluence api url"
    )
    parser.add_argument(
        "--confluence_user", type=str, default=None, help="Confluence user"
    )
    parser.add_argument(
        "--confluence_password", type=str, default=None, help="Confluence password"
    )
    parser.add_argument(
        "--dump_source",
        type=str,
        default=None,
        help="Dump graphviz source to directory",
    )
    parser.add_argument(
        "--cacert",
        type=str,
        default=None,
        help="If you need to specify a CA certificate file",
    )

    return parser.parse_args()


def run() -> None:
    """Main program loop"""

    args = parse_args()

    dm = DataModel(
        args.url, args.http_username, args.http_password, args.uid, args.cacert
    )
    dm.load()

    if dm.status != 200:
        print("{} Status code {}".format(str(datetime.datetime.now()), dm.status))
        return

    try:
        old_dm = pickle.load(open("cache.dat", "rb"))
        if old_dm == dm:
            return
    except FileNotFoundError:
        print("First run")

    print("{} Graphing changes".format(str(datetime.datetime.now())))

    pickle.dump(dm, open("cache.dat", "wb"))

    dot = graphviz.Digraph(comment="Double edge facts")

    # for obj in dm.objects:
    #     dot.node(obj, obj)

    for fact in dm.facts:
        name, s, d, di = fact
        if name == "mentions":
            continue
        if not d:
            continue

        dot.node(s, s)
        dot.node(d, d)
        if di:
            dot.edge(s, d, label=name, dir="both")
        else:
            dot.edge(s, d, label=name)

    if args.dump_source:
        with open(os.path.join(args.dump_source, "double.dot"), "w") as f:
            f.write(dot.source)

    dot.render("output/double", format="png", renderer="cairo")

    dot = graphviz.Digraph(comment="Single edge facts")

    for fact in dm.facts:

        name, s, d, di = fact

        if d:
            continue

        dot.node(name, label=name, shape="diamond")
        dot.node(s, s)

        dot.edge(s, name)

    if args.dump_source:
        with open(os.path.join(args.dump_source, "single.dot"), "w") as f:
            f.write(dot.source)

    dot.render("output/single", format="png", renderer="cairo")

    dot = graphviz.Digraph(comment="All Double edge facts")

    for obj in dm.objects:
        dot.node(obj, obj)

    for fact in dm.facts:
        name, s, d, di = fact
        if not d:
            continue

        dot.node(s, s)
        dot.node(d, d)
        if di:
            dot.edge(s, d, label=name, dir="both")
        else:
            dot.edge(s, d, label=name)

    if args.dump_source:
        with open(os.path.join(args.dump_source, "complete.dot"), "w") as f:
            f.write(dot.source)

    dot.render("output/complete", format="png", renderer="cairo")

    if args.parent_id:
        try:
            os.environ.pop("https_proxy")
        except KeyError:
            pass
        try:
            os.environ.pop("http_proxy")
        except KeyError:
            pass

        verify_ssl = False if args.cacert else True

        confluence = Confluence(
            url=args.confluence_url,
            username=args.confluence_user,
            password=args.confluence_password,
            verify_ssl=verify_ssl,
        )
        confluence.attach_file(
            "output/double.cairo.png",
            page_id=args.parent_id,
            title="Double Edged Facts",
        )
        confluence.attach_file(
            "output/single.cairo.png",
            page_id=args.parent_id,
            title="Single Edged Facts",
        )
        confluence.attach_file(
            "output/complete.cairo.png",
            page_id=args.parent_id,
            title="Single Edged Facts",
        )

        if args.dump_source:
            confluence.attach_file(
                os.path.join(args.dump_source, "complete.dot"),
                page_id=args.parent_id,
                title="complete source",
            )
            confluence.attach_file(
                os.path.join(args.dump_source, "double.dot"),
                page_id=args.parent_id,
                title="double source",
            )
            confluence.attach_file(
                os.path.join(args.dump_source, "single.dot"),
                page_id=args.parent_id,
                title="single source",
            )


if __name__ == "__main__":
    run()



#!/usr/bin/env python3

""" Add ACT types """

import functools
import json
import os
import re
from typing import Any, Dict, List, Text

from pkg_resources import resource_filename

DEFAULT_VALIDATOR = r"(.|\n)*"


class TypeLoadError(Exception):
    pass


def etc_file(filename: Text) -> Text:
    "Get content of file from resource/etc"
    return resource_filename(
        "act.types", "etc/{}".format(filename)
    )  # ).decode('utf-8')


@functools.lru_cache(32)
def default_object_types():
    return load_types(etc_file("object-types.json"))


def default_fact_types():
    return load_types(etc_file("fact-types.json"))


def default_meta_fact_types():
    return load_types(etc_file("meta-fact-types.json"))


@functools.lru_cache(32)
def get_object_validator(object_type: Text) -> Text:
    """Get object validator, default to DEFAULT_VALIDATOR)"""

    typedef = [t for t in default_object_types() if t["name"] == object_type]

    if not typedef:
        raise TypeLoadError(f"Type not found: {object_type}")
    if len(typedef) > 1:
        print(typedef)
        raise TypeLoadError(f"Multiple types found for {object_type}")

    return typedef[0].get("validator", DEFAULT_VALIDATOR)


def object_validates(object_type: Text, object_value: Text) -> bool:
    """Validate object using current valdiator"""

    if not isinstance(object_value, Text):
        raise TypeError(
            "Illegal type for argument object_value: "
            f"(object_value={object_value}, object_value (type) = {type(object_value)}, object_type={object_type})"
        )

    if object_value == "*":
        # Fact Chain value -> do not validate
        return True

    validator = get_object_validator(object_type)

    if re.fullmatch(validator, object_value):
        return True

    return False


def load_types(filename: Text) -> List[Dict[Text, Any]]:
    """
    Parse as json, and exit on parse error
    """
    if not os.path.isfile(filename):
        raise TypeLoadError(f"File not found: {filename}")

    try:
        with open(filename, encoding="UTF8") as f:
            types = json.load(f)
    except json.decoder.JSONDecodeError:
        raise TypeLoadError(f"Unable to parse file as json: {filename}")

    if not isinstance(types, list):
        raise TypeLoadError("Types must be a list of dicts")

    return types



import re
import string
from typing import Text


def default_lowercase_format(value: Text) -> Text:
    """Default lowercase format
    * remove text in paranthesis
    * replace multiple whitespace with single space
    * lowercase
    """

    return re.sub(r"\s+", " ", re.sub(r"\(.*\)", "", str(value))).lower().strip()


def hash_format(value: Text) -> Text:
    value = value.strip()

    if (value.upper() == value) and all(c in string.hexdigits for c in value):
        # All characters in the hash is uppercase hexdigists so it is safe to lowercase
        # the value We can not always lowercase, since we support hashes with mixed
        # case, e.g. ssdeep
        return value.lower()

    return value


OBJECT_FORMATTERS = {
    "tool": default_lowercase_format,
    "threatActor": default_lowercase_format,
    "vulnerability": default_lowercase_format,
    "hash": hash_format,
    "content": hash_format,
}


def object_format(object_type: Text, object_value: Text) -> Text:
    if object_type in OBJECT_FORMATTERS:
        return OBJECT_FORMATTERS[object_type](object_value)

    # No formatter specified
    return object_value
